unit ChildEventUnit;

interface

uses
  SysUtils, Classes, ChildMainFormUnit;

type
  TTestChildEvent = class(TThread)
  private
    FUIString: String;
    //Two routines to simplify showing our progress
    procedure UpdateUI;
    procedure WriteString(const S: String);
  protected
    //Body of the thread
    procedure Execute; override;
    //This relies on the event having happened
    procedure CodeReliantOnTheEvent;
  public
    constructor Create;
    destructor Destroy; override;
  end;

implementation

uses
  Windows, IPCThrd;

const
  EventName = 'SampleEvent';

constructor TTestChildEvent.Create;
begin
  inherited Create(False);
  WriteString(Format('Thread %x started', [ThreadID]))
end;

destructor TTestChildEvent.Destroy;
begin
  WriteString(Format('Thread %x terminated', [ThreadID]));
  inherited Destroy
end;

procedure TTestChildEvent.UpdateUI;
begin
  Form1.Listbox1.Items.Add(FUIString)
end;

procedure TTestChildEvent.WriteString(const S: String);
begin
  FUIString := S;
  Synchronize(UpdateUI);
end;

procedure TTestChildEvent.Execute;
var
  LocalEvent: TEvent;
begin
  try
    LocalEvent := TEvent.Create(EventName, False)
  except
    WriteString('Event already created')
  end;
  try
    WriteString(Format(
      'Thread %x waiting for event', [ThreadID]));
    if LocalEvent.Wait(Infinite) then
      CodeReliantOnTheEvent;
  finally
    LocalEvent.Free
  end
end;

procedure TTestChildEvent.CodeReliantOnTheEvent;
begin
  WriteString(Format(
    'Thread %x has seen the event', [ThreadID]));
end;

end.