unit ParentMutexUnit;

interface

uses
  Classes, ParentMainFormUnit;

type
  TTestParentMutex = class(TThread)
  private
    FUIString: String;
    //Two routines to simplify showing our progress
    procedure UpdateUI;
    procedure WriteString(const S: String);
  protected
    //Body of the thread
    procedure Execute; override;
    //This relies on mutually exclusive access
    procedure ThisNeedsMutualExclusiveAccess;
  public
    constructor Create;
    destructor Destroy; override;
  end;

implementation

uses
  SysUtils, Windows;

const
  MutexName = 'SampleMutex';

var
  Mutex: THandle;

constructor TTestParentMutex.Create;
begin
  inherited Create(False);
  WriteString(Format('Thread %x started', [ThreadID]))
end;

destructor TTestParentMutex.Destroy;
begin
  WriteString(Format('Thread %x terminated', [ThreadID]));
  inherited Destroy
end;

procedure TTestParentMutex.UpdateUI;
begin
  Form1.Listbox1.Items.Add(FUIString)
end;

procedure TTestParentMutex.WriteString(const S: String);
begin
  FUIString := S;
  Synchronize(UpdateUI);
end;

procedure TTestParentMutex.Execute;
begin
  WriteString(Format(
    'Thread %x waiting for mutex', [ThreadID]));
  WaitForSingleObject(Mutex, Infinite);
  ThisNeedsMutualExclusiveAccess;
  ReleaseMutex(Mutex);
end;

procedure TTestParentMutex.ThisNeedsMutualExclusiveAccess;
begin
  WriteString(Format(
    'Thread %x got mutex', [ThreadID]));
  Sleep(4000);
  WriteString(Format(
    'Thread %x finished with mutex', [ThreadID]));
end;

initialization
  Mutex := CreateMutex(nil, False, MutexName)
finalization
  //This is not strictly necessary as all open
  //ThreadIDs are closed by the Win32 OS upon termination
  if Mutex <> 0 then
    CloseHandle(Mutex)
end.
