unit MutexUnit;

interface

uses
  Classes, MainFormUnit;

type
  TTestMutex = class(TThread)
  private
    FUIString: String;
    //Two routines to simplify showing our progress
    procedure UpdateUI;
    procedure WriteString(const S: String);
  protected
    //Body of the thread
    procedure Execute; override;
    //This relies on mutually exclusive access
    procedure ThisNeedsMutualExclusiveAccess;
  public
    constructor Create;
    destructor Destroy; override;
  end;

implementation

uses
  SysUtils, Windows, IPCThrd;

const
  MutexName = 'SampleMutex';

var
  Mutex: TMutex;

constructor TTestMutex.Create;
begin
  inherited Create(False);
  WriteString(Format('Thread %x started', [Handle]))
end;

destructor TTestMutex.Destroy;
begin
  WriteString(Format('Thread %x terminated', [Handle]));
  inherited Destroy
end;

procedure TTestMutex.UpdateUI;
begin
  Form1.Listbox1.Items.Add(FUIString)
end;

procedure TTestMutex.WriteString(const S: String);
begin
  FUIString := S;
  Synchronize(UpdateUI);
end;

procedure TTestMutex.Execute;
begin
  WriteString(Format(
    'Thread %x waiting for mutex', [Handle]));
  if Mutex.Get(Infinite) then
    try
      ThisNeedsMutualExclusiveAccess
    finally
      Mutex.Release
    end
  else
    WriteString(Format(
      'Thread %x failed to get mutex', [Handle]));
end;

procedure TTestMutex.ThisNeedsMutualExclusiveAccess;
begin
  WriteString(Format(
    'Thread %x got mutex', [Handle]));
  Sleep(4000);
  WriteString(Format(
    'Thread %x finished with mutex', [Handle]));
end;

initialization
  Mutex := TMutex.Create(MutexName)
finalization
  Mutex.Free
end.
